// file: $JEIDA/src/signal_detector/v2.1/src/sigd_main_0.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "signal_detector.h"
#include "signal_detector_constants.h"

// define a format for outputing endpoint information
//
#define SIGD_OUTPUT_FORMAT "%4ld beg_time = %9.3f secs   end_time = %9.3f secs\n"

// define the grow size of the endpoints buffer
//
#define SIGD_ENDPTS_GROW_SIZE 10000

// define parameters used in postprocessing the endpoints
//
#define SIGD_MAX_GAP_TIME 2.0
#define SIGD_INITIAL_PAD_TIME 0.54
#define SIGD_FINAL_PAD_TIME 0.50

// local functions
//
extern int_4 grow_arrays_cc(float_4** arrays, int_4 num_arrays,
			    int_4& cur_size, int_4 grow_size);
extern logical_1 adjust_endpoints_cc(float_4** endpts, int_4 size,
				     int_4 num_endpts, float_4 gap_time,
				     float_4 initial_pad_time,
				     float_4 final_pad_time);

//-----------------------------------------------------------------------------
//
// program: signal_detector
// synopsis: signal_detector.exe [options] files
// descr: automatically locates utterances in a file
// example: signal_detector file.param signal.raw signal.endpoints
//
// options: none
//
// arguments:
//  file.param: (input) parameter file
//  signal.raw: (input) binary signal file
//  signal.endpoints: (output) contains beg and end times of utterances
//
//-----------------------------------------------------------------------------
main(int argc, char **argv) {

  // declare an output buffer
  //
  int_4 endpts_max_size = SIGD_ENDPTS_GROW_SIZE;
  float_4* endpts[2];
  endpts[(int_4)0] = new float_4[endpts_max_size];
  endpts[(int_4)1] = new float_4[endpts_max_size];

  // declare signal-related objects
  //
  Signal_detector sig_det;

  //---------------------------------------------------------------------------
  //
  // initialization
  //
  //---------------------------------------------------------------------------

  // load parameters
  //
  if (sig_det.load_parameters_cc((char_1*)argv[1]) == ISIP_FALSE) {
    fprintf(stdout, "%s: error reading parameters (%s)\n", argv[0], argv[1]);
    return(ISIP_ERROR);
  }
  sig_det.debug_cc(stdout, (char_1*)"signal_detector_main");

  // open the input file
  //
  FILE* fp_in = fopen(argv[2], "r");

  if (fp_in == NULL) {
    fprintf(stdout, "%s: error opening input file (%s)\n", argv[0], argv[2]);
    return(ISIP_ERROR);
  }

  // open the output file
  //
  FILE* fp_out = fopen(argv[3], "w");

  if (fp_out == NULL) {
    fprintf(stdout, "%s: error opening output file (%s)\n", argv[0], argv[3]);
    return(ISIP_ERROR);
  }

  // get the amount of data to read per i/o call
  //
  int_4 nbytes_in = sig_det.get_nbytes_to_read_cc();
  fprintf(stdout, "input buffer size = %ld bytes\n", nbytes_in);

  uint_1* signal_buf = new uint_1[nbytes_in];
  memset(signal_buf, (int_4)0, nbytes_in);

  // initialize the signal detector
  //
  if (sig_det.initialize_cc() == ISIP_FALSE) {
    fprintf(stdout, "%s: error during initialization\n", argv[0]);
    return(ISIP_ERROR);
  }

  //---------------------------------------------------------------------------
  //
  // main processing loop
  //
  //---------------------------------------------------------------------------

  // loop over the data
  //
  int_4 nf = (int_4)0;
  int_4 num_utt = (int_4)0;
  float_4 t1, t2;

  while (fread(signal_buf, (int_4)1, nbytes_in, fp_in) > (int_4)0) {

    // process data
    //
    if (sig_det.process_cc((void*)signal_buf) == ISIP_FALSE) {
      fprintf(stdout, "%s: error processing frame no. %ld\n", argv[0], nf);
      return(ISIP_ERROR);
    }

    // check for an utterance
    //
    if (sig_det.get_endpoints_cc(t1, t2) == ISIP_TRUE) {

      // make sure we have space for the new endpts
      //
      if (num_utt == endpts_max_size) {
	grow_arrays_cc(endpts, (int_4)2,
		       endpts_max_size, SIGD_ENDPTS_GROW_SIZE);
      }

      // save the endpoints
      //
      (endpts[(int_4)0])[num_utt] = t1;
      (endpts[(int_4)1])[num_utt] = t2;

      // send some output to the display
      //
      fprintf(stdout, SIGD_OUTPUT_FORMAT, num_utt,
	      (endpts[(int_4)0])[num_utt], (endpts[(int_4)1])[num_utt]);

      // increment counters
      //
      num_utt++;
      sig_det.reset_cc();
    }

    // increment counters
    //
    memset(signal_buf, (int_4)0, nbytes_in);
    nf++;
  }

  //---------------------------------------------------------------------------
  //
  // cleanup
  //
  //---------------------------------------------------------------------------

  // flush the signal detector
  //
  if (sig_det.flush_cc() == ISIP_FALSE) {
    fprintf(stdout, "%s: error flushing signal detector\n", argv[0]);
    return(ISIP_ERROR);
  }

  // check one last time for an utterance
  //
  if (sig_det.get_endpoints_cc(t1, t2) == ISIP_TRUE) {

    // make sure we have space for the new endpts
    //
    if (num_utt == endpts_max_size) {
      grow_arrays_cc(endpts, (int_4)2,
		     endpts_max_size, SIGD_ENDPTS_GROW_SIZE);
    }

    // save the endpoints
    //
    (endpts[(int_4)0])[num_utt] = t1;
    (endpts[(int_4)1])[num_utt] = t2;

    // send some output to the display
    //
    fprintf(stdout, SIGD_OUTPUT_FORMAT, num_utt,
	    (endpts[(int_4)0])[num_utt], (endpts[(int_4)1])[num_utt]);

    // increment counters
    //
    num_utt++;
    sig_det.reset_cc();
  }

  //---------------------------------------------------------------------------
  //
  // postprocessing
  //
  //---------------------------------------------------------------------------

  // adjust the endpoints:
  //  break utterances midway between the each endpoint and the following
  //  startpoint, unless this is too big
  //
  adjust_endpoints_cc(endpts, (int_4)2, num_utt, SIGD_MAX_GAP_TIME,
		      SIGD_INITIAL_PAD_TIME, SIGD_FINAL_PAD_TIME);

  // output these endpoints
  //
  for (int_4 utt_num=(int_4)0; utt_num<num_utt; utt_num++) {
    fprintf(fp_out, SIGD_OUTPUT_FORMAT, utt_num,
	    (endpts[(int_4)0])[utt_num], (endpts[(int_4)1])[utt_num]);
  }

  //---------------------------------------------------------------------------
  //
  // exit
  //
  //---------------------------------------------------------------------------

  // close files
  //
  fclose(fp_in);
  fclose(fp_out);

  // exit gracefully
  //
  return(ISIP_NO_ERROR);
}

